/* БАЗОВЫЕ КОНТЕКСТЫ (основные стили поведения) - contexts.go

1	Пищевой	- Пищевое поведение, восполнение энергии, на что тратится время и тормозятся антагонистические стили поведения.
2	Поиск	- Поисковое поведение, любопытство. Обследование объекта внимания, поиск новых возможностей.
3	Игра	- Игровое поведение - отработка опыта в облегченных ситуациях или при обучении.
4	Гон	- Половое поведение. Тормозятся антагонистические стили
5	Защита	- Оборонительные поведение для явных признаков угрозы или плохом состоянии.
6	Лень	- Апатия в благополучном или безысходном состоянии.
7	Ступор	- Оцепенелость при непреодолимой опастности или когда нет мотивации при благополучии или отсуствии любых возможностей для активного поведения.
8	Страх	- Осторожность при признаках опасной ситуации.
9	Агрессия	- Агрессивное поведение для признаков легкой добычи или защиты (иногда - при плохом состоянии).
10	Злость	- Безжалостность в случае низкой оценки .
11	Доброта	- Альтруистическое поведение.
12	Сон - Состояние сна. Освобождение стрессового состояния. Реконструкция необработанной информации.
 */
package gomeostas

import (
	"BOT/lib"
	"sort"
	"strconv"
	"strings"
)


func init(){

}

/* масссив активностей базовых контекстов
активный - true, неактивный - false
 */
var BaseContextActive =make(map[int]bool)
var BaseContextWeight =make(map[int]int)

/* Прошивка несовместимых сочетаний контекстов
Для каждого основного контекста - антагонисты
 */
var antagonists =make(map[int][]int)

func initContextDetector(){
	antagonists[1]=[]int{7, 5, 12}
	antagonists[2]=[]int{5, 6, 7, 9, 12}
	antagonists[3]=[]int{6, 7, 12}
	antagonists[4]=[]int{7, 12}
	antagonists[5]=[]int{2, 6, 7, 12}
	antagonists[6]=[]int{2, 3, 5, 7, 12}
	antagonists[7]=[]int{1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12}
	antagonists[8]=[]int{7, 9, 12}
	antagonists[9]=[]int{5, 7, 11, 12}
	antagonists[10]=[]int{7, 11, 12}
	antagonists[11]=[]int{7, 9, 10, 12}
	antagonists[12]=[]int{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11}

	path:=lib.GetMainPathExeFile()
	lines,_:=lib.ReadLines(path+"/memory_reflex/base_context_weight.txt")
	for i := 0; i < len(lines); i++ {
		p:=strings.Split(lines[i], "|")
		id,_:=strconv.Atoi(p[0])
		val,_:=strconv.Atoi(p[1])
		BaseContextWeight[id]=val
	}

	for id, _ := range BaseContextWeight {
		BaseContextActive[id]=false
	}
}
////////////////
/* состояние базового контекста зависит
1) от выхода из нормы жизненных параметров
2) от безусловно прошитых признаков восприятия
Антагонисты конкурируют между собой со своими весами значимости.

!!! BaseContextActive[6]=true - Ступор регулируется как отсуствие реакций в данном контексте при опасности
 */
func baseContextUpdate(){
// сначала все контексты выключены:
	for id, _ := range BaseContextWeight {
		BaseContextActive[id]=false
	}

// энергия
	if BadNormalWell[1]==1 { //плохо
		BaseContextActive[1]=true // Пищевой
		BaseContextActive[2]=true //Поиск
	}
	if BadNormalWell[1]==3 { // хорошо
		BaseContextActive[11]=true //	Доброта
	}
	// Стресс
	if BadNormalWell[2]==1 { //плохо
		BaseContextActive[2]=true //Поиск
		BaseContextActive[5]=true // Защита
		BaseContextActive[8]=true // Страх
		BaseContextActive[3]=false //Игра
		BaseContextActive[4]=false //	Гон
		BaseContextActive[6]=false // 	Лень
		BaseContextActive[11]=false //	Доброта
	}
	// Гон
	if BadNormalWell[3]==1 { //плохо, невтерпеж
		BaseContextActive[2]=true //Поиск
		BaseContextActive[6]=false // 	Лень
		BaseContextActive[1]=false // Пищевой
	}
	// Потребность в общении
	if BadNormalWell[4]==1 { //плохо, невыносимо одиноко
		BaseContextActive[2]=true //Поиск
		BaseContextActive[3]=true //Игра
		BaseContextActive[11]=true // Доброта
		BaseContextActive[6]=false // Лень
	}
	// Потребность в обучении
	if BadNormalWell[5]==1 { //плохо, очень хочется стать лучше, научиться
		BaseContextActive[2]=true //Поиск
		BaseContextActive[3]=true //Игра
		BaseContextActive[6]=false // Лень
	}
	// Поиск
	if BadNormalWell[6]==1 { //плохо, очень хочется узнать больше
		BaseContextActive[2]=true //Поиск
		BaseContextActive[3]=true //Игра
		BaseContextActive[6]=false // Лень
		BaseContextActive[9]=false // Агрессия
		BaseContextActive[10]=false //	Злость
	}
	// Жадность
	if BadNormalWell[7]==1 { //плохо, острая нехватка, самосохранение
		BaseContextActive[2]=true //Поиск
		BaseContextActive[5]=true //Защита
		BaseContextActive[8]=true //Страх
		BaseContextActive[9]=true // Агрессия
		BaseContextActive[3]=false //Игра
		BaseContextActive[11]=false //	Доброта
	}

// Конкурентность антагонистов
	keys := make([]int, 0, len(BaseContextActive))
	for id, v := range BaseContextActive {
		if v {
			keys = append(keys, id)
		}
	}
	sort.Ints(keys)
	for _, id := range keys {
		for _, ida := range antagonists[id] {
			if BaseContextActive[ida] && BaseContextWeight[ida] > BaseContextWeight[id]{// активный антагонист значимее
			BaseContextActive[id]=false // погасить текущий контекст
			break // больше не нужно смотреть других антагонистов
				}else{
				BaseContextActive[ida]=false // погасить антогониста
			}
			}
	}
}
/////////////////////////////////////////////////////



// для Пульта
func GetCurContextActive()(string) {
	var out=""
	for id, v := range BaseContextActive {
		if(v){
			out+=strconv.Itoa(id)+";1|"
		}else{
			out+=strconv.Itoa(id)+";0|"
		}
	}
	return out
}
/////////////////////////////////////////////////////////

// контекст распознавания текущей фразы с Пульта для Vernike_detector.go
func GetActiveContextInfo()(map[int]int){
	var activeCW =make(map[int]int)
	if BaseContextActive==nil || len(BaseContextActive)<2{
		return nil
	}
	keys := make([]int, 0, len(BaseContextActive))
	for id, v := range BaseContextActive {
		if v {
			keys = append(keys, id)
		}
	}
	sort.Ints(keys)
	for _, k := range keys {
		activeCW[k]=BaseContextWeight[k]
	}
	return activeCW
}